screen chapter_select:

    # Any screen that has "use chapter_select" should have the following
    # screen attributes:
    #
    # tag menu
    # So that Return() replaces the screen with the main menu.
    #
    # zorder -2
    # So that the screen has lower zorder than the other chapter select
    # screens (gui and bg).  That way, this screen can hide the other chapter
    # select screens with an on 'hide' event.
    # (Yes, there is something weird when you have higher zorder:
    # http://lemmasoft.renai.us/forums/viewtopic.php?f=8&t=18631 )

    on "show" action [
        Show('chapter_select_bg', bg=ChapterSelectImages.default_background),
        Show('chapter_select_gui',
            chapter_sequence=chapter_sequence,
            ChapterSelectImages=ChapterSelectImages,
        ),
    ]
    on "replace" action [
        Show('chapter_select_bg', bg=ChapterSelectImages.default_background),
        Show('chapter_select_gui',
            chapter_sequence=chapter_sequence,
            ChapterSelectImages=ChapterSelectImages,
        ),
    ]

    on "hide" action [
        Hide('chapter_select_bg'),
        Hide('chapter_select_gui'),
    ]
    on "replaced" action [
        Hide('chapter_select_bg'),
        Hide('chapter_select_gui'),
    ]


screen story_chapter_select:

    tag menu
    zorder -2

    use chapter_select(
        # Structure holding the chapters
        chapter_sequence=story_chapters,
        # Chapter images class
        ChapterSelectImages=StoryChapterSelectImages,
    )


screen chapter_select_gui:
    
    imagemap:
        ground ChapterSelectImages.ground
        idle ChapterSelectImages.exit_idle
        hover ChapterSelectImages.exit_hover
        
        # Exit button
        hotspot (71, 417, 133, 40) clicked Return()

        
    # Chapter select box
    
    $ BUTTON_SIZE = (44, 35)
    
    window:
        area (18, 109, 475, 94)
        xpadding 20 ypadding 22
        background None
        
        hbox:
            style_group "chapter_list"
            spacing 20

            for chapter in chapter_sequence.get_chapter_list():

                if chapter.is_unlocked():
                    # Click on this button to see the chapter details.
                    textbutton ("{0}".format(chapter.number)) action ActionAndSelect(
                        SelectChapter(chapter, ChapterSelectImages), chapter.number
                    ):
                        minimum BUTTON_SIZE
                        maximum BUTTON_SIZE
                        
                else:
                    # The chapter isn't unlocked.  Have a placeholder
                    # element instead.
                    frame:
                        background None
                        minimum BUTTON_SIZE
                        maximum BUTTON_SIZE
                                
    on "hide" action Hide("chapter_select_gui_2")
    on "replaced" action Hide("chapter_select_gui_2")
        
    
screen chapter_select_gui_2:

    # Chapter info box
    window:
        background None
        area (26, 258, 457, 146)
        text chapter.title:
            style "chapter_title_text"
            xalign 0.5 ypos 10
        text chapter.description:
            style "description_text"
            xpos 5 ypos 66
            
    # GO! button
    button:
        area (270, 411, 176, 50)
        clicked StartChapter(chapter.start_label)
        idle_background ChapterSelectImages.go_idle
        hover_background ChapterSelectImages.go_hover
        


screen chapter_select_bg:

    # Put this behind the main chapter select screen.
    zorder -1

    add bg
    
    

# This screen should be called with a transition.
# That way, the screen makes the menus transition to black (since this screen
# is all black), then calls the Start label after a time delay.

screen chapter_select_black_fg_then_start:

    # Put this in front of all the chapter select screens.  Use it to get a
    # fade-to-black effect when you start a chapter.
    zorder 1

    # Make sure the buttons of the below screens aren't clickable or
    # hoverable anymore.
    modal True

    add 'bg solid black'

    # 0.5 is the length of a dissolve.
    timer 0.5 action Start(label)



init python:
    
    from ordereddict import OrderedDict
    
    class StoryChapterSelectImages(object):
        """
        Filenames of images used in the Chapter Start menu
        (which is for story chapters).
        """
        prefix = "gui_images/chapter_select/"
        
        ground = prefix + "chapter_select_ground.png"
        default_background = Solid("#000000")
        new = prefix + "chapter_new.png"
        complete = prefix + "chapter_select_complete.png"
        exit_idle = prefix + "exit_idle.png"
        exit_hover = prefix + "exit_hovered.png"
        go_idle = prefix + "go_idle.png"
        go_hover = prefix + "go_hovered.png"
        chapter_button_selected_bg = prefix + "selected_chapter_box.png"
        ch1_splash = prefix + "chap1_splash.png"
        ch2_splash = prefix + "chap2_splash.png"
        
    ##
    # Styling for the chapter select screens.
    ##
    
    style.chapter_list_button = Style(style.default, properties=dict(
        idle_background = None,
        hover_background = None,
        selected_idle_background = StoryChapterSelectImages.chapter_button_selected_bg,
        selected_hover_background = StoryChapterSelectImages.chapter_button_selected_bg,
    ))
    style.chapter_list_button_text = Style(style.default, properties=dict(
        font = "fonts/BABYU___.TTF",
        size = 25,
        idle_color = "#9CB2B0",
        hover_color = "#ffffff",
        pos = (8, 3),
    ))
    
    style.chapter_title_text = Style(style.default, properties=dict(
        size = 25,
    ))
    
    ##
    # Screen actions.
    ##

    class SelectChapter(Action):

        def __init__(self, chapter, ChapterSelectImages):

            self.chapter = chapter
            self.ChapterSelectImages = ChapterSelectImages

        def __call__(self):

            Show("chapter_select_gui_2",
                chapter=self.chapter,
                ChapterSelectImages=self.ChapterSelectImages,
            )()
            Show("chapter_select_bg",
                bg=self.chapter.preview_image,
                transition=dissolve,
            )()

    class StartChapter(Action):
        
        def __init__(self, label):

            self.label = label

        def __call__(self):

            # Play a sound.
            renpy.sound.play(se_boost1)

            # Call the screen chapter_select_black_fg_then_start.
            # This screen dissolves to black by dissolving in an all-black
            # screen, then calls the Start label after a time delay.
            #
            # Why do we need a hackish way of dissolving to black?  Start() is
            # pretty much an exception call that stops everything and instantly
            # hides all menus (without transitions).  All this happens before
            # the start label is called, so anything we put after the start
            # label to dissolve to black would be useless.
            Show('chapter_select_black_fg_then_start',
                label=self.label,
                transition=dissolve,
            )()

    ##
    # Chapter completion statuses.
    ##
    
    # If this is the first time running the game, create the dictionary of
    # chapter statuses.
    #
    # Don't bother initializing the chapter statuses themselves at this point.
    # Reason: if the chapter statuses are initialized (to "new") in version 2.0,
    # and the same copy of the game is later upgraded to version 3.0,
    # the chapter 3 status will still need to be initialized.
    if not persistent.chapter_status:
        persistent.chapter_status = dict()
    
    ##
    # Chapter info.
    #
    # When writing chapter descriptions, be sure to test them in-game to make
    # sure the description fits in the box.
    ##
    
    class ChapterStatus(object):
        """
        Chapter status constants.
        """
        NEW = "new"
        STARTED = "started"
        COMPLETE = "complete"
        
    class ChapterSequence(object):
        """
        A sequence of chapters.
        For example, the story chapters can be one sequence, and the
        Hellcat Corner scenes can be one sequence.
        """
        
        def __init__(self):
            self._chapters = OrderedDict()
            
            # Not just chapter numbers, but 'last chapter completed' statuses.
            # None is the 'completed no chapters' status.
            self._chapter_numbers = [None]
            
        def add_chapter(self, chapter):
            self._chapters[chapter.number] = chapter
            self._chapter_numbers.append(chapter.number)
    
        def get_chapter(self, number):
            return self._chapters[number]

        def get_chapter_list(self):
            """
            Return a list of the chapters, in order from first to last.

            They're in order because they were added to the chapters
            OrderedDict in order.
            """
            return self._chapters.values()

        def number_compare_key(self, n):
            if not n in self._chapter_numbers:
                raise ValueError("{0} is not an existing chapter number.".format(n))
            return self._chapter_numbers.index(n)

        def latest_cleared_chapter(self):
            """
            Return the number of the latest cleared chapter.
            """
            # Check the chapters in reverse order (last to first), and as
            # soon as we see a completed chapter, return that.
            for chapter in self.get_chapter_list()[::-1]:
                if chapter.get_status() == ChapterStatus.COMPLETE:
                    return chapter.number
            # No chapters completed.
            return None

        def has_cleared_chapter(self, chapter_number):
            """
            Returns True if the chapter is considered 'cleared' for the
            purposes of unlocking extras; False otherwise.
            """
            latest_cleared = self.latest_cleared_chapter()

            # If latest_cleared >= chapter_number, then chapter_number is considered cleared.
            return self.number_compare_key(latest_cleared) >= self.number_compare_key(chapter_number)

    class Chapter(object):
        """
        A single chapter.
        For example, chapter 1 of the story.

        This is an abstract base class; use a subclass of this class to
        actually instantiate a Chapter.
        """
        
        def __init__(self, number, title, description, preview_image,
                     start_label):
            self.number = number
            self.title = title
            self.description = description
            self.preview_image = preview_image
            self.start_label = start_label

        def get_status(self):
            """
            Returns the status of this chapter.
            
            Also initializes this chapter's status if necessary.

            TODO: Track statuses separately for each set of chapters.
            This isn't actually an issue if we don't care about Hellcat
            Corner completion status, though.
            (If we were to implement this, then probably some kind of prefix
            would be used for the chapter_status variable or key, with a
            different prefix for each set of chapters.)
            """
            if not persistent.chapter_status.has_key(self.number):
                persistent.chapter_status[self.number] = ChapterStatus.NEW
                return ChapterStatus.NEW
            else:
                return persistent.chapter_status[self.number]
    
        def update_status(self, new_status):
            if new_status == ChapterStatus.STARTED:
                if self.get_status() == ChapterStatus.NEW:
                    persistent.chapter_status[self.number] = ChapterStatus.STARTED
            elif new_status == ChapterStatus.COMPLETE:
                persistent.chapter_status[self.number] = ChapterStatus.COMPLETE

        def is_unlocked(self):
            # Subclasses should implement this method.
            pass
        
    class StoryChapter(Chapter):
        def is_unlocked(self):
            # All story chapters are unlocked from the start.
            return True
        
    
    story_chapters = ChapterSequence()
    
    story_chapters.add_chapter(StoryChapter(
        number = '1',
        title = "Chapter 1: The Fledgling Sun God",
        description = """Utsuho Reiuji travels to the Remains of Blazing Hell to attain the power of the sun god Yatagarasu.""",
        preview_image = StoryChapterSelectImages.ch1_splash,
        start_label = "Chp_1_1",
    ))
        
    story_chapters.add_chapter(StoryChapter(
        number = '2',
        title = "Chapter 2: Defending Chireiden",
        description = """The Moriya-Chireiden Alliance braces for attacks from both Parsia and the new unknowns.""",
        preview_image = StoryChapterSelectImages.ch2_splash,
        start_label = "Chp_2_1",
    ))
    