screen database:
    
    tag menu
    
    default database_entries_to_display = init_database_entries_to_display()
    default database_entry_lookup = init_database_entry_lookup()
    
    default database_mode = 'list'
    default current_tab = 'mecha'
    default current_entry = None
    default sort_method = 'faction'
    default recent_pages = []
    default main_viewport_key = 'database list mecha'
    
    # Ground, windows, exit button, sort, and category tabs
    
    imagemap:
        alpha False
        # TODO: Consider making these composite images on init, with image statements?
        ground im.Composite((1024,576),
            (0,0), DatabaseImages.ground,
            (0,0), DatabaseImages.windows[current_tab],
        )
        idle DatabaseImages.idle
        hover DatabaseImages.hover
        selected_idle DatabaseImages.selected_idle
        selected_hover DatabaseImages.selected_hover
        
        # Exit button
        hotspot (17, 11, 105, 32) clicked Return()
        
        # Back button
        if  database_mode == 'article':
            hotspot (643, 510, 69, 32) action database_back_to_list
        else:
            hotspot (643, 510, 69, 32) clicked None
            
        # Category tabs
        hotspot (396, 71, 109, 50) clicked DatabaseSwitchTab('mecha')
        hotspot (515, 71, 163, 50) clicked DatabaseSwitchTab('characters')
        hotspot (688, 71, 140, 50) clicked DatabaseSwitchTab('glossary')
        
        # Sort buttons
        hotspot (745, 13, 107, 30) action SetDatabaseSortMethod("alpha")
        hotspot (858, 13, 107, 30) action SetDatabaseSortMethod("faction")
        
    # Upper-left pane with info and image
    use database_info
    
    # Main window
    window:
        area (400, 136, 560, 359)
        xpadding 0 ypadding 0
        background None
        
        viewport id main_viewport_key:
            yadjustment initialize_adjustment(
                main_viewport_key,
                persistent_save=True,
            )
            
            mousewheel True
                
            if database_mode == 'list':
                frame:
                    left_padding 20 right_padding 53 ypadding 16
                    background None
                    
                    if sort_method == "alpha":
                    
                        vbox:
                            style_group "database_article_list"
                            for entry in database_entries_by_alpha(current_tab):
                                use database_list_item
                                    
                    elif sort_method == "faction":
                        
                        vbox:
                            style_group "database_faction_list"
                            for faction, faction_entries in database_entries_by_faction(current_tab).iteritems():
                                text faction:
                                    xpos (62+15)    # Line up with article names...
                                
                                vbox:
                                    style_group "database_article_list"
                                    for entry in faction_entries:
                                        use database_list_item
                                    
                                
            elif database_mode == 'article':
                frame:
                    left_padding 31 right_padding 64 ypadding 16
                    background None
                    
                    text current_entry.article_text:
                        style "database_article_text"
        
    # Main window's scrollbar
    vbox:
        xpos 926 ypos 133
        
        # Amount scrolled when the up or down button is clicked.  Make it equal to 3 lines.
        if database_mode == 'list':
            $ scroll_amount = 99
        elif database_mode == 'article':
            $ scroll_amount = 66
        
        style_group "database_main_scrollbar"
        
        # TODO: perhaps make hovered versions of the up/down buttons.
        # TODO: make holding down the mouse on the up/down buttons scroll repeatedly.
        button action ScrollUp(main_viewport_key, scroll_amount):
            style "database_main_scrollbar_arrow_up"
            
        vbar value YScrollValue(main_viewport_key)
        
        button action ScrollDown(main_viewport_key, scroll_amount):
            style "database_main_scrollbar_arrow_down"
    
    # Recent Pages box
    window:
        area (58, 443, 316, 99)
        
        left_padding 27 right_padding 7 top_padding 0 bottom_padding 0
        background None
        
        viewport id "database_recent_pages":
            yadjustment initialize_adjustment(
                "database_recent_pages",
                persistent_save=True,
            )
            
            mousewheel True
                
            vbox:
                style_group "database_recent_list"
                for entry_name in recent_pages[::-1]:
                    $ entry = database_entry_lookup[entry_name]
                    textbutton entry_name:
                        text_style "database_recent_list_button_text_{0}".format(entry.category)
                        clicked ShowDatabaseArticle(entry, from_recent_pages_link=True)
            
    # Recent Pages scrollbar
    vbox:
        xpos 345 ypos 443
        
        # Amount scrolled when the up or down button is clicked.  Make it equal to 1 line.
        $ scroll_amount = 22
        
        style_group "database_recent_scrollbar"
        
        # TODO: perhaps make hovered versions of the up/down buttons.
        # TODO: make holding down the mouse on the up/down buttons scroll repeatedly.
        button action ScrollUp("database_recent_pages", scroll_amount):
            style "database_recent_scrollbar_arrow_up"
            
        vbar value YScrollValue("database_recent_pages")
        
        button action ScrollDown("database_recent_pages", scroll_amount):
            style "database_recent_scrollbar_arrow_down"
            
    on "show" action Play('music', 'music/oc-tm-library.mp3', fadeout=1.0)
    on "replace" action Play('music', 'music/oc-tm-library.mp3', fadeout=1.0)
    
screen database_list_item:
    
    hbox:
        spacing 15
        window:
            minimum (62, 12) maximum (62, 12)
            xpadding 0 ypadding 0
            yalign 0.3
            
            $ status = entry.get_status()
            if status == 'new':
                add DatabaseImages.new xalign 1.0
            elif status == 'updated':
                add DatabaseImages.updated xalign 1.0
            
        textbutton entry.name:
            hovered SetScreenVariable("current_entry", entry)
            clicked ShowDatabaseArticle(entry)
    
screen database_info:
    
    if current_entry:
    
        window:
            area (55, 80, 317, 308)
            left_padding 19 right_padding 18 ypadding 16
            background None
            
            if database_mode == 'list':
                $ info_text = current_entry.short_description
            elif database_mode == 'article':
                if current_entry.category == 'mecha':
                    $ info_text = current_entry.full_name
                else:
                    $ info_text = current_entry.name
                
            text info_text yalign 0.0:
                style "database_info_text"
            add (DatabaseImages.article_images_prefix + current_entry.image_file) yalign 1.0
    
init python:
    
    from ordereddict import OrderedDict
    
    class DatabaseImages(object):
        """
        Filenames of images used in the database screens.
        """
        prefix = "gui_images/database/"
        article_images_prefix = prefix + "article_images/"
        
        ground = prefix + "database_ground.png"
        windows = dict(
            mecha = prefix + "database_mecha_windows.png",
            characters = prefix + "database_characters_windows.png",
            glossary = prefix + "database_glossary_windows.png",
        )
        idle = prefix + "database_idle.png"
        hover = prefix + "database_hover.png"
        selected_idle = prefix + "database_selected_idle.png"
        selected_hover = prefix + "database_selected_hover.png"
        
        new = prefix + "database_new.png"
        updated = prefix + "database_updated.png"
    
    style.database_info_text = Style(style.default, properties=dict(
        # The indent properties prevent italic text from getting cut off on
        # the left side.
        first_indent = 5,
        rest_indent = 5,
    ))
    style.database_article_list_vbox = Style(style.default, properties=dict(
        spacing = 11,
    ))
    style.database_article_list_button = Style(style.default, properties=dict(
        background = None,
    ))
    style.database_article_list_button_text = Style(style.default, properties=dict(
        size = 22,
        idle_color = "#9bb1af",
        hover_color = "#ffffff",
        # The indent properties prevent italic text from getting cut off on
        # the left side.
        first_indent = 5,
        rest_indent = 5,
    ))
    
    style.database_faction_list_vbox = Style(style.default, properties=dict(
        spacing = 24,
    ))
    style.database_faction_list_text = Style(style.default, properties=dict(
        size = 22,
        font = "fonts/BABYU___.TTF",
        color = "#e4e195",
        # The indent properties prevent italic text from getting cut off on
        # the left side.
        first_indent = 5,
        rest_indent = 5,
    ))
    
    style.database_article_text = Style(style.default, properties=dict(
        size = 18,
        line_spacing = 4,
        # The indent properties prevent italic text from getting cut off on
        # the left side.
        first_indent = 5,
        rest_indent = 5,
    ))
    
    style.database_recent_list_vbox = Style(style.default, properties=dict(
        spacing = 5,
    ))
    style.database_recent_list_button = Style(style.default, properties=dict(
        background = None,
    ))
    style.database_recent_list_button_text = Style(style.default, properties=dict(
        size = 18,
        hover_color = "#ffffff",
        # The indent properties prevent italic text from getting cut off on
        # the left side.
        first_indent = 5,
        rest_indent = 5,
    ))
    style.database_recent_list_button_text_mecha = Style(style.database_recent_list_button_text, properties=dict(
        idle_color = "#b19b9d",
    ))
    style.database_recent_list_button_text_characters = Style(style.database_recent_list_button_text, properties=dict(
        idle_color = "#9bb1a1",
    ))
    style.database_recent_list_button_text_glossary = Style(style.database_recent_list_button_text, properties=dict(
        idle_color = "#9bb1af",
    ))
    
    style.database_main_scrollbar_vscrollbar = ScrollbarStyle(width=21, height=324)
    style.database_main_scrollbar_arrow_up = ScrollbarArrowStyle(direction='up', width=21)
    style.database_main_scrollbar_arrow_down = ScrollbarArrowStyle(direction='down', width=21)
    
    style.database_recent_scrollbar_vscrollbar = ScrollbarStyle(width=21, height=60)
    style.database_recent_scrollbar_arrow_up = ScrollbarArrowStyle(direction='up', width=21)
    style.database_recent_scrollbar_arrow_down = ScrollbarArrowStyle(direction='down', width=21)
    
    class DatabaseSwitchTab(Action):
        def __init__(self, tab_name):
            self.tab_name = tab_name
            
        def __call__(self):
            if get_screen_variable("database_mode") == "article":
                add_to_recent_pages(get_screen_variable("current_entry").name)
                set_screen_variable("database_mode", "list")
            set_screen_variable("current_tab", self.tab_name)
            set_screen_variable("current_entry", None)
            
            update_database_main_viewport()
            
        def get_selected(self):
            return get_screen_variable("current_tab") == self.tab_name
                
    class ShowDatabaseArticle(Action):
        def __init__(self, entry, from_recent_pages_link=False):
            self.entry = entry
            self.from_recent_pages_link = from_recent_pages_link
            
        def __call__(self):
            current_entry = get_screen_variable("current_entry")
            if self.from_recent_pages_link:
                if (get_screen_variable("database_mode") == "article"
                   and current_entry.name != self.entry.name):
                    # We're exiting another article, so add that article we're exiting
                    # to the recent pages list.
                    add_to_recent_pages(current_entry.name)
                if get_screen_variable("current_tab") != self.entry.category:
                    # We're going to a recent page in another category, so we
                    # should change the category tab.
                    set_screen_variable("current_tab", self.entry.category)
    
            self.entry.mark_as_read()
            
            set_screen_variable("current_entry", self.entry)
            set_screen_variable("database_mode", "article")
            
            update_database_main_viewport()
            
    class SetDatabaseSortMethod(Action):
        def __init__(self, sort_method):
            self.sort_method = sort_method
            
        def __call__(self):
            set_screen_variable("sort_method", self.sort_method)
    
            # Reset category lists' viewport values to 0.  Reasons:
            # (1) Retaining the same scroll position isn't too useful
            # after switching the sort method.
            # (2) When switching from faction to alpha, the fact that the
            # viewport contents shrink (due to no more faction headers) can
            # make the viewport get scrolled beyond its bottom, which looks
            # a bit odd.
    
            # Reset the current category list's viewport value to 0.
            current_viewport_key = get_screen_variable("main_viewport_key")
            YScrollValue(current_viewport_key).get_adjustment().change(0)
    
            # Reset each category list's saved viewport value to 0.
            viewport_positions = persistent.viewport_positions
            viewport_positions['database list mecha'] = 0
            viewport_positions['database list characters'] = 0
            viewport_positions['database list glossary'] = 0
            
        def get_selected(self):
            return get_screen_variable("sort_method") == self.sort_method
            
    def database_back_to_list():
        current_entry = get_screen_variable("current_entry")
        add_to_recent_pages(current_entry.name)
        
        set_screen_variable("database_mode", "list")
        update_database_main_viewport()
        
    def add_to_recent_pages(entry_name):
        recent_pages = get_screen_variable("recent_pages")
        
        # Add the entry to the recent pages list.
        if entry_name in recent_pages:
            # Entry is already in recent_pages, so "bump" it up to the top
            # by removing it from its current position and re-appending it.
            recent_pages.remove(entry_name)
        recent_pages.append(entry_name)
                
        # Check to see if the recent pages list needs trimming.
        MAX_RECENT_PAGES = 10
        if len(recent_pages) > MAX_RECENT_PAGES:
            # Remove the least recently visited page from the list.
            del recent_pages[0]
            
    def update_database_main_viewport():
        database_mode = get_screen_variable("database_mode")
        current_tab = get_screen_variable("current_tab")
        current_entry = get_screen_variable("current_entry")
        
        # Update the viewport key
        if database_mode == "list":
            key = "database {0} {1}".format(database_mode, current_tab)
        elif database_mode == "article":
            key = "database {0} {1}".format(database_mode, current_entry.name)
            
        set_screen_variable("main_viewport_key", key)
        
    def init_database_entries_to_display():
        """
        Get the database entries which will actually display on the database screen.
        
        This means including only the database entries that have been unlocked,
        and using the image file, article text, etc. appropriate for the
        player's level of progress.
        """
        database_entries_to_display = dict()
        for tab in database_entries.keys():
            database_entries_to_display[tab] = [e.get_unlocked_entry() for e in database_entries[tab] if e.is_unlocked()]
        return database_entries_to_display

    def database_entries_by_alpha(tab):
        return sorted(get_screen_variable("database_entries_to_display")[tab], key=attrgetter('name'))
        
    def database_entries_by_faction(tab):
        unsorted_entries = get_screen_variable("database_entries_to_display")[tab]
        entries_by_faction = OrderedDict()
        
        for faction in Factions.ordered:
            faction_entries = [e for e in unsorted_entries if e.faction == faction]
            if faction_entries:
                entries_by_faction[faction] = faction_entries
            
        return entries_by_faction
        
    def init_database_entry_lookup():
        database_entry_lookup = dict()
        entries = get_screen_variable("database_entries_to_display")
        for category_name, category_entries in entries.iteritems():
            for entry in category_entries:
                database_entry_lookup[entry.name] = entry
        return database_entry_lookup
        