# Save, Load
#
# Screens that allow the user to save and load the game.
# http://www.renpy.org/doc/html/screen_special.html#save
# http://www.renpy.org/doc/html/screen_special.html#load

# Since saving and loading are so similar, we combine them into
# a single screen, file_picker. We then use the file_picker screen
# from simple load and save screens.

screen file_picker:

    add "gui_images/Save_ground.png"

    frame:
        style_group "file_picker"
        xalign 0.48
        
        has hbox
                
        viewport id "file picker viewport":
            maximum (720,334)
            ypos 18

            yadjustment initialize_adjustment("file picker viewport", persistent_save=True)
            
            mousewheel True
            
            $ columns = 2
            $ rows = 12
                
            # Display a grid of file slots.
            grid columns rows:
                style_group "file_picker"
                spacing 6
                
                transpose False    # Row by row
                
                # For loop going through each file slot.
                for i in range(1, columns * rows + 1):
    
                    # Each file slot is a button.
                    button:
                        minimum (344, 108)
                        top_padding 9
                        left_padding 13

                        action FileAction(i)
    
                        has hbox:
                            style "file_picker_button_hbox"
                            spacing 16
                            
                        # Add the screenshot.
                        add FileScreenshot(i) 
                        
                        # Format the description, and add it as text.
                        $ description = "%s.\n%s\n%s" % (
                            FileSlotName(i, columns * rows),
                            FileTime(i, empty=_("unused")),
                            FileSaveName(i))
    
                        text description:
                            # Positioning the text with a nonzero xpos MAY
                            # make the right side of the text get cut off,
                            # depending on the hbox spacing.  It's easier to
                            # just do the x-positioning solely with hbox
                            # spacing, instead of this property.
                            pos (0,10)
    
                        key "save_delete" action FileDelete(i)
                    
        vbox:
            ypos 20
            
            style_group "save_scrollbar"
            
            # TODO: perhaps make hovered versions of the up/down buttons.
            # TODO: make holding down the mouse on the up/down buttons scroll repeatedly.
            button action ScrollUp("file picker viewport", 114):
                style "save_scrollbar_arrow_up"
                
            vbar value YScrollValue("file picker viewport")
            
            button action ScrollDown("file picker viewport", 114):
                style "save_scrollbar_arrow_down"
                    
                    
screen save:

    # This ensures that any other submenu screen is replaced.
    tag submenu
    
    # Make sure the say screen (dialogue box) never shows above menus.
    zorder 1

    use file_picker

screen load:

    # This ensures that any other submenu screen is replaced.
    tag submenu
    
    # Make sure the say screen (dialogue box) never shows above menus.
    zorder 1

    use file_picker

init python:
    SAVELOAD_COLOR_1 = "#9CB2B0"
    
    config.thumbnail_width = 160
    config.thumbnail_height = 90
    
    style.file_picker_frame = Style(style.menu_frame)
    style.file_picker_frame.background = None
    
    style.file_picker_viewport = Style(style.default)
    style.file_picker_viewport.clipping = True    # This prevents the viewport from spilling out of its area
    
    # Why is it vscrollbar?  The widget is called vbar...
    # Well, that's what the style system calls it, so can't do much about it.
    style.save_scrollbar_vscrollbar = ScrollbarStyle(width=21, height=295)
    
    style.save_scrollbar_arrow_up = ScrollbarArrowStyle(direction='up', width=21)
    style.save_scrollbar_arrow_down = ScrollbarArrowStyle(direction='down', width=21)
    
    style.file_picker_grid = Style(style.default)

    style.file_picker_nav_button = Style(style.small_button)
    style.file_picker_nav_button_text = Style(style.small_button_text)

    style.file_picker_button = Style(style.large_button)
    style.file_picker_button.idle_background = "gui_images/Save_slot_idle.png"
    style.file_picker_button.hover_background = "gui_images/Save_slot_hovered.png"
    style.file_picker_button.insensitive_background = "gui_images/Save_slot_unused.png"
    style.file_picker_button_hbox = Style(style.file_picker_hbox)
    
    style.file_picker_text = Style(style.large_button_text)
    style.file_picker_text.font = "fonts/maass slicer Italic.ttf"
    style.file_picker_text.color = SAVELOAD_COLOR_1
    style.file_picker_text.size = 20
    # The indent properties prevent italic text from getting cut off on
    # the left side.
    style.file_picker_text.first_indent = 5
    style.file_picker_text.rest_indent = 5
    