# Say screen
#
# Screen that's used to display adv-mode dialogue.
# http://www.renpy.org/doc/html/screen_special.html#say

screen say:

    # Defaults for side_image and two_window
    default side_image = None
    default two_window = True

    # Decide if we want to use the one-window or two-window varaint.
    if not two_window:

        fixed:
            style "say_two_window_vbox"
            # The one window variant.
            window:
                yoffset -40
                id "window"
    
                has vbox:
                    style "say_vbox"
    
                text what id "what"

    else:

        # The two window variant.
        # Use fixed instead of hbox, so one window's position doesn't affect the other window's position
            fixed:
                style "say_two_window_vbox"

                # Dialogue/narration
                # This window seems to automatically use the style "say_window"
                window:
                    id "window"
    
                    has vbox:
                        style "say_vbox"
    
                    text what id "what"
                    
                # Name window
                window:
                    style "say_who_window"
                    
                    if who:
                        text who:
                            id "who"
                    else:
                        text ""    # If we didn't add this empty text, then the name box would expand like crazy
                        
    # If there's a side image, display it above the text.
    if side_image:
        add side_image
    else:
        add SideImage() xalign 0.0 yalign 1.0

    # Use the quick menu.
    use quick_menu
        
##############################################################################
# Quick Menu
#
# A screen that's included by the default say screen, and adds quick access to
# several useful functions.
screen quick_menu:

    # Add an in-game quick menu.
    imagemap:
        ground "gui_images/option_bar_blank.png"
        idle "gui_images/text_box_buttons_imagemap_idle.png"
        hover "gui_images/text_box_buttons_imagemap_hovered.png"
        selected_idle "gui_images/text_box_buttons_imagemap_idle_selected.png"
        selected_hover "gui_images/text_box_buttons_imagemap_hovered_selected.png"
        
        # Menu
        # Before showing the game menu, take a screenshot to be used if the
        # game is saved.  This way, the savefile screenshot won't include the game menu.
        hotspot (8, 6, 52, 23) clicked [ FileTakeScreenshot(), Show("game_menu") ]

        # QS
        hotspot (63, 6, 52, 23) clicked quicksave_handler    # See quicksave.rpy
             
        # Auto
        hotspot (118, 6, 52, 23) clicked Preference("auto-forward", "toggle")

        ## Skip
        ## TODO: Implement the functionality for Skip-hold, or just remove the Hold option
        hotspot (173, 6, 52, 23) clicked Skip()

        # Hide
        hotspot (228, 6, 52, 23) clicked renpy.curried_call_in_new_context("_hide_windows")
        style_group "quick"
    
        xalign 1.0
        yalign 1.0

        
init -2 python:
    style.quick_button.set_parent('default')
    style.quick_button.background = None
    style.quick_button.xpadding = 5

    style.quick_button_text.set_parent('default')
    style.quick_button_text.size = 12
    style.quick_button_text.idle_color = "#8888"
    style.quick_button_text.hover_color = "#ccc"
    style.quick_button_text.selected_idle_color = "#cc08"
    style.quick_button_text.selected_hover_color = "#cc0"
    style.quick_button_text.insensitive_color = "#4448"
    
    # Set a default value for the auto-forward time, and note that AFM is
    # turned off by default.
    config.default_afm_time = 10
    config.default_afm_enable = False
    
    
##############################################################################
# Say screen styling.
init python:
    ## The background of the window. In a Frame, the two numbers
    ## are the size of the left/right and top/bottom borders,
    ## respectively.

    style.say_window.background = "gui_images/text_box.png"
    style.say_window.ypos = 205
    style.say_window.ymaximum = 167
    style.say_window.yminimum = 167
    
    ## Margin is space surrounding the window, where the background
    ## is not drawn.

    style.say_window.left_margin = 45
    style.say_window.right_margin = 45
    # style.say_window.top_margin = 0
    style.say_window.bottom_margin = 0

    ## Padding is space inside the window, where the background is
    ## drawn.

    style.say_window.left_padding = 48
    style.say_window.right_padding = 48
    style.say_window.top_padding = 64

    ## This is the minimum height of the window, including the margins
    ## and padding.

    style.say_window.yminimum = 167


    ## Now for the name box
    style.say_who_window.background = None
    style.say_who_window.xminimum = 276
    style.say_who_window.yminimum = 34
    
    style.say_who_window.xpos = 71
    style.say_who_window.ypos = -95
    
    style.say_who_window.xmargin = 0
    style.say_who_window.ymargin = 0
    style.say_who_window.left_padding = 55
    style.say_who_window.right_padding = 0
    style.say_who_window.top_padding = 17    # Alternatively, have no top/bottom padding and specify the text's positioning as yalign=0.5
    style.say_who_window.bottom_padding = 0
    
    
    
    ## Button box
    
    style.say_button_window = Style(style.window)
    style.say_button_window.background = None
    style.say_button_window.xminimum = 288
    style.say_button_window.xmaximum = 288
    style.say_button_window.yminimum = 33
    style.say_button_window.ymaximum = 33
    
    style.say_button_window.xpos = 830
    style.say_button_window.yalign = 1.0
    
    style.say_button_window.xmargin = 0
    style.say_button_window.ymargin = 0
    style.say_button_window.xpadding = 0
    style.say_button_window.ypadding = 0
