﻿# This file is in the public domain. Feel free to modify it as a basis
# for your own screens.

# Generic style-related stuff.
# TODO: More screens need to use init python (0 priority) instead of -2 for their style code.
# TODO: More screens need to use these classes/functions.
init -1 python:
        
    class ScrollbarStyle(Style):
        BASE_WIDTH = 21
        BASE_SCROLLBAR_HEIGHT = 295
        BASE_THUMB_HEIGHT = 22
        SCROLLBAR_IMAGE = "gui_images/scrollbar/scrollbar_background.png"
        THUMB_IMAGE = "gui_images/scrollbar/scrollbar_thumb.png"
        
        def __init__(self, width, height):
            super(ScrollbarStyle, self).__init__(style.vbar)
                
            if height > ScrollbarStyle.BASE_SCROLLBAR_HEIGHT:
                top_bar = bottom_bar = LiveTile(ScrollbarStyle.SCROLLBAR_IMAGE)
            else:
                top_bar = bottom_bar = ScrollbarStyle.SCROLLBAR_IMAGE
            
            self.properties.extend([
                dict(bar_invert = True),
                dict(top_bar = top_bar),
                dict(bottom_bar = bottom_bar),
                dict(thumb = ScrollbarStyle.THUMB_IMAGE),
                dict(thumb_offset = 0),
                dict(top_gutter = 2),    # The gutter is the space where the thumb isn't allowed to scroll
                dict(bottom_gutter = 2),
                dict(ypos = 0),
                dict(maximum = (width, height)),
            ])
            
    class ScrollbarArrowStyle(Style):
        BASE_WIDTH = 21
        BASE_ARROW_HEIGHT = 19
        UP_ARROW_IMAGE = "gui_images/scrollbar/scrollbar_up_arrow.png"
        DOWN_ARROW_IMAGE = "gui_images/scrollbar/scrollbar_down_arrow.png"
        
        def __init__(self, direction, width):
            super(ScrollbarArrowStyle, self).__init__(style.default)
            
            if direction == 'up':
                background = ScrollbarArrowStyle.UP_ARROW_IMAGE
            elif direction == 'down':
                background = ScrollbarArrowStyle.DOWN_ARROW_IMAGE
            else:
                background = "#f00"    # Invalid direction
            
            self.properties.extend([
                dict(background = background),
                dict(xpadding = 0),
                dict(ypadding = 0),
                dict(xmargin = 0),
                dict(ymargin = 0),
                dict(maximum = (width, ScrollbarArrowStyle.BASE_ARROW_HEIGHT)),
            ])
    
    # The outer window of the Save/Load, Options, and Chapter Select menus.
    style.menu_outer_window = Style(style.menu_window, properties=dict(
        
        # Background is taken care of by an image, so don't need to specify it here.
        # However, if you want to test that the area is correct, temporarily set this to a semi-transparent color (like "#f008").
        background = None,
        
        # (xpos, ypos, width, height)
        area = (44, 15, 710, 357),
    ))
    
    # The description text in the chapter select screen and extras screen.
    # TODO: Apply this to the in-game menu screen too?
    style.description_text = Style(style.default, properties=dict(
        size = 18,
        line_spacing = 4,
    ))
    style.footnote_text = Style(style.default, properties=dict(
        color = "#f7f67f",
        size = 16,
        line_spacing = 4,
    ))

##############################################################################
# Say
#
# Screen that's used to display adv-mode dialogue.
# http://www.renpy.org/doc/html/screen_special.html#say

# **Moved to say_screen.rpy**


##############################################################################
# Choice
#
# Screen that's used to display in-game menus.
# http://www.renpy.org/doc/html/screen_special.html#choice

screen choice:

    window: 
        style "menu_window"        
        xalign 0.5
        yalign 0.5
        
        vbox:
            style "menu"
            spacing 2
            
            for caption, action, chosen in items:
                
                if action:  
                    
                    button:
                        action action
                        style "menu_choice_button"                        

                        text caption style "menu_choice"
                    
                else:
                    text caption style "menu_caption"

init -2 python:
    config.narrator_menu = True
    
    style.menu_window.set_parent(style.default)
    style.menu_choice.set_parent(style.button_text)
    style.menu_choice.clear()
    style.menu_choice_button.set_parent(style.button)
    style.menu_choice_button.xminimum = int(config.screen_width * 0.75)
    style.menu_choice_button.xmaximum = int(config.screen_width * 0.75)


##############################################################################
# Input
#
# Screen that's used to display renpy.input()
# http://www.renpy.org/doc/html/screen_special.html#input

screen input:

    window:
        has vbox

        text prompt
        input id "input"
        
    use quick_menu

        
##############################################################################
# Nvl
#
# Screen used for nvl-mode dialogue and menus.
# http://www.renpy.org/doc/html/screen_special.html#nvl

screen nvl:

    window:
        style "nvl_window"

        has vbox:
            style "nvl_vbox"

        # Display dialogue.
        for who, what, who_id, what_id, window_id in dialogue:
            window:
                id window_id

                has hbox:
                    spacing 10

                if who is not None:
                    text who id who_id

                text what id what_id

        # Display a menu, if given.
        if items:

            vbox:
                id "menu"

                for caption, action, chosen in items:

                    if action:

                        button:
                            style "nvl_menu_choice_button"
                            action action

                            text caption style "nvl_menu_choice"

                    else:

                        text caption style "nvl_dialogue"

    add SideImage() xalign 0.0 yalign 1.0
    
    use quick_menu
        
##############################################################################
# Main Menu 
#
# Screen that's used to display the main menu, when Ren'Py first starts
# http://www.renpy.org/doc/html/screen_special.html#main-menu

# **Moved to main_menu_screen.rpy**


##############################################################################
# Navigation
#
# Screen that's included in other screens to display the game menu
# navigation and background.
# http://www.renpy.org/doc/html/screen_special.html#navigation
screen navigation:

    # The background of the game menu.
    window:
        style "gm_root"

    # The various buttons.
    frame:
        style_group "gm_nav"
        xalign .98
        yalign .98
        
        has vbox

        textbutton _("Return") action Return()
        textbutton _("Preferences") action ShowMenu("preferences")
        textbutton _("Save Game") action ShowMenu("save")
        textbutton _("Load Game") action ShowMenu("load")
        textbutton _("Main Menu") action MainMenu()
        textbutton _("Help") action Help()
        textbutton _("Quit") action Quit()

init -2 python:
    style.gm_nav_button.size_group = "gm_nav"
    

##############################################################################
# Save, Load
#
# Screens that allow the user to save and load the game.
# http://www.renpy.org/doc/html/screen_special.html#save
# http://www.renpy.org/doc/html/screen_special.html#load

# Since saving and loading are so similar, we combine them into
# a single screen, file_picker. We then use the file_picker screen
# from simple load and save screens.
    
# **Moved to save_load_screens.rpy**
    

##############################################################################
# Preferences
#
# Screen that allows the user to change the preferences.
# http://www.renpy.org/doc/html/screen_special.html#prefereces
    
# **Moved to settings_screen.rpy**


##############################################################################
# Yes/No Prompt
#
# Screen that asks the user a yes or no question.
# http://www.renpy.org/doc/html/screen_special.html#yesno-prompt
    
# **Moved to yesno_prompt_screen.rpy**


##############################################################################
# Debug screen
#
# To use this, append some debug info to store.debug_lines, and display it
# within a screen with:
# use debug_display
#
screen debug_display:
    text debug_lines_display():
        xpos 0 ypos 0
        color "#0f0c"
        size 18
        
init python:
    store.debug_lines = []
    def debug_lines_display():
        while len(store.debug_lines) > 4:
            del store.debug_lines[0]
        return '\n'.join(store.debug_lines)
    
    
##############################################################################
# Utility functions for screens

init python:

    def get_screen_variable(name):
        cs = renpy.current_screen()
        if cs is not None:
            if cs.scope.has_key(name):
                return cs.scope[name]
        return None
    
    def set_screen_variable(name, value):
        SetScreenVariable(name, value)()
        
    def set_variable(name, value):
        SetVariable(name, value)()
        


##############################################################################
# Screen Actions
#
# See common/00screen.rpy for the default Actions.  Here we'll define custom Actions.

init python:
    
    class NullAction(Action):
        """
        An action that does nothing.
        For example, if you want to disable a certain key
        (say, hide_windows) in a screen, make that key map to this action.
        """
        def __call__(self):
            pass
    
    class ActionAndSelect(Action):
        """
        Runs an action and makes the clicked button
        change to the "selected" style.
    
        Parameters:
        action - the action to run.
        action_name - a unique name for the button.
    
        Inner workings: uses a variable "selected_name" which is in the
        scope of the current screen.  This is set to the action_name of
        the currently selected button.
        """
        
        def __init__(self, action, action_name):
            self.action = action
            self.action_name = action_name
            
        def __call__(self):
            self.action()
            SetScreenVariable("selected_name", self.action_name)()
            
        def get_selected(self):
            screen_scope = renpy.current_screen().scope
            if screen_scope.has_key("selected_name"):
                return screen_scope["selected_name"] == self.action_name
            else:
                return False
    
    class ActionAndToggleSelect(Action):
        """
        Runs an action and makes the clicked button
        toggle its "selected" status.
    
        Parameters:
        action - the action to run.
        action_name - a unique name for the button.
    
        Inner workings: uses a variable "selected_name" which is in the
        scope of the current screen.  This is set to the action_name of
        the currently selected button.
        """
        
        def __init__(self, action, action_name):
            self.action = action
            self.action_name = action_name
            
        def __call__(self):
            self.action()
            screen_scope = renpy.current_screen().scope
            if screen_scope.has_key("selected_name") and screen_scope["selected_name"] == self.action_name:
                SetScreenVariable("selected_name", None)()
            else:
                SetScreenVariable("selected_name", self.action_name)()
            
        def get_selected(self):
            screen_scope = renpy.current_screen().scope
            if screen_scope.has_key("selected_name"):
                return screen_scope["selected_name"] == self.action_name
            else:
                return False
        
    
    class OnViewportChange(Action):
        """
        Class for a 'changed' callback for a viewport whose scroll position is to be saved.
        When the viewport position changes, the __call__ function is run.
        """
        def __init__(self, viewport_key, persistent_save):
            self.viewport_key = viewport_key
            self.persistent_save = persistent_save
            
        def __call__(self, value):
            if self.persistent_save:
                viewport_positions = persistent.viewport_positions
            else:
                viewport_positions = get_screen_variable('viewport_positions')
            # Save the viewport position.
            viewport_positions[self.viewport_key] = value
        
    def initialize_adjustment(viewport_key, persistent_save=False):
        """
        Returns the yadjustment property for a viewport whose scroll position is to be saved.
        
        viewport_key must be a unique string for each viewport, so that different
        viewports' positions can be saved separately.

        ***** NOTE *****
        As of 2012 Dec 28 + Ren'Py 6.14.1.366, it seems that using
        persistent_save=False makes Touhou Mecha screens use unbounded CPU
        and memory.  The fix: use persistent_save=True instead.  No idea why
        this is the case.
        """
        if persistent_save:
            # viewport_positions is in a persistent variable.
            viewport_positions = persistent.viewport_positions
            if viewport_positions is None:
                viewport_positions = dict()
                persistent.viewport_positions = viewport_positions
        else:
            # viewport_positions is in a screen variable.
            viewport_positions = get_screen_variable('viewport_positions')
            if viewport_positions is None:
                viewport_positions = dict()
                set_screen_variable('viewport_positions', viewport_positions)
        
        # If this viewport has a saved value, load that value.
        # If not, start the viewport at position 0.
        if viewport_positions.has_key(viewport_key):
            initial_value = viewport_positions[viewport_key]
        else:
            initial_value = 0
               
        return ui.adjustment(value=initial_value, changed=OnViewportChange(viewport_key, persistent_save))
    
    
    class ScrollUp(Action):
        """
        Scroll a viewport up by a fixed amount.
        """
        
        def __init__(self, viewport, amount):
            self.yscrollvalue = YScrollValue(viewport)
            self.amount = amount  # Amount to scroll by
            
        def __call__(self):
            # Get the Adjustment object (behavior.py, class Adjustment)
            yadjustment = self.yscrollvalue.get_adjustment()
            # Use change() instead of set_value() to do bounds checking
            yadjustment.change(yadjustment.get_value() - self.amount)
    
    class ScrollDown(Action):
        """
        Scroll a viewport down by a fixed amount.
        """
        
        def __init__(self, viewport, amount):
            self.yscrollvalue = YScrollValue(viewport)
            self.amount = amount  # Amount to scroll by
            
        def __call__(self):
            # Get the Adjustment object (behavior.py, class Adjustment)
            yadjustment = self.yscrollvalue.get_adjustment()
            # Use change() instead of set_value() to do bounds checking
            yadjustment.change(yadjustment.get_value() + self.amount)
            
    
    class MainMenuCustom(MainMenu):
        """
        Returns to the Main Menu.
        
        Use this instead of the MainMenu action from 00screen.rpy.
        Reasons follow:
        
        MainMenu's get_sensitive() method returns False when the current game
        context is the main menu.  Right after you've selected "Yes" to the
        "Are you sure you want to return to the main menu" prompt, the context
        becomes the main menu.  As a result, during the fadeout, the main menu
        button becomes insensitive.  We don't really have a use for this
        behavior (no way to accidentally click the main menu button while
        returning to the main menu), and this behavior introduces a graphical
        oddity: we don't have insensitive styles defined for the main menu
        button, so the button doesn't display during the fadeout.
        
        This action, on the other hand, always considers the main menu button
        to be "sensitive".  So the button does display during fadeout.
        """

        def get_sensitive(self):
            return True
