﻿## This file contains some of the options that can be changed to customize
## your Ren'Py game. It only contains the most common options... there
## is quite a bit more customization you can do.
##
## Lines beginning with two '#' marks are comments, and you shouldn't
## uncomment them. Lines beginning with a single '#' mark are
## commented-out code, and you may want to uncomment them when
## appropriate.

init -1 python hide:

    ## Should we enable the use of developer tools? This should be
    ## set to False before the game is released, so the user can't
    ## cheat using developer tools.

    config.developer = True

    ## These control the width and height of the screen.

    config.screen_width = 1024
    config.screen_height = 576

    ## If not None, this is expected to be the filename of an image giving an icon that is used for the window on Windows.
    ## This should be a 32x32 image with 1-bit alpha. (Opaque images work the best.)
    
    config.windows_icon = "gui_images/tm_icon.png"
    
    ## This controls the title of the window, when Ren'Py is
    ## running in a window.

    config.window_title = u"Touhou Mecha 2.0"

    # These control the name and version of the game, that are reported
    # with tracebacks and other debugging logs.
    config.name = "Touhou Mecha"
    config.version = "2.0"
    
    config.skip_delay = 25
    
    # Allow fast-skip to the next choice menu (with >) while developer mode is off?
    config.fast_skipping = False
    
    # Disable fast-skip even for developer mode... in most cases.
    # It'll still work if you hide the window and then press >,
    # or click "X" to bring up the quit-game prompt and then press >.
    def fast_skip_disabler():
        ui.keymap(fast_skip=ui.returns(None))
    config.overlay_functions.append(fast_skip_disabler)
    
    # Action when you click the window close button ("X" on Windows).
    # Make it behave exactly like the "Quit" menu option: if in main menu, then
    # quit without prompting, and otherwise, quit prompt appears over everything else.
    def custom_quit():
        if renpy.context()._main_menu:
            Quit(confirm=False)()
        else:
            Quit()()
    config.quit_action = custom_quit
    
    #Makes the dialogue box stay during transitions.
    
    #config.empty_window = renpy.curry(extend)("", interact=False)
    #_last_say_what = ""
    
    #########################################
    # Themes
    
    ## We then want to call a theme function. themes.roundrect is
    ## a theme that features the use of rounded rectangles. It's
    ## the only theme we currently support.
    ##
    ## The theme function takes a number of parameters that can
    ## customize the color scheme.

    theme.tv(
        # Color scheme: White Chocolate
                                    
        ## The color of an idle widget face.
        widget = "#33271C",

        ## The color of a focused widget face.
        widget_hover = "#ECE7C4",

        ## The color of the text in a widget.
        widget_text = "#B99D83",

        ## The color of the text in a selected widget. (For
        ## example, the current value of a preference.)
        widget_selected = "#ffffff",

        ## The color of a disabled widget face. 
        disabled = "#614D3A",

        ## The color of disabled widget text.
        disabled_text = "#80654D",

        ## The color of informational labels.
        label = "#F1EBE5",

        ## The color of a frame containing widgets.
        frame = "#926841",

        ## The background of the main menu. This can be a color
        ## beginning with '#', or an image filename. The latter
        ## should take up the full height and width of the screen.
        mm_root = "#FBF9EA",

        ## The background of the game menu. This can be a color
        ## beginning with '#', or an image filename. The latter
        ## should take up the full height and width of the screen.
        gm_root = "#FBF9EA",

        ## If this is True, the in-game window is rounded. If False,
        ## the in-game window is square.
        rounded_window = False,

        ## And we're done with the theme. The theme will customize
        ## various styles, so if we want to change them, we should
        ## do so below.            
        )
    
    
    #########################################
    ## This lets you change the placement of the main menu.

    ## The way placement works is that we find an anchor point
    ## inside a displayable, and a position (pos) point on the
    ## screen. We then place the displayable so the two points are
    ## at the same place.

    ## An anchor/pos can be given as an integer or a floating point
    ## number. If an integer, the number is interpreted as a number
    ## of pixels from the upper-left corner. If a floating point,
    ## the number is interpreted as a fraction of the size of the
    ## displayable or screen.

    #style.mm_menu_frame.xpos = 0.0
    #style.mm_menu_frame.xanchor = 0.0
    #style.mm_menu_frame.ypos = 0.5
    #style.mm_menu_frame.yanchor = 0.5


    #########################################
    ## These let you customize the default font used for text in Ren'Py.

    ## The file containing the default font.

    style.default.font = "fonts/Enigmatic_Regular.ttf"
    ## Set the font's italics as a .ttf (don't let Ren'Py use its generic italics conversion)
    config.font_replacement_map["fonts/Enigmatic_Regular.ttf", False, True] = ("fonts/Enigma_2i.TTF", False, False)

    ## The default size of text.

    style.default.size = 22
    #Increase the spacing to keep the tops of tall text characters from being "previewed" by the line above
    style.default.line_leading = 1

    ## Note that these only change the size of some of the text. Other
    ## buttons have their own styles.


    #########################################
    ## These settings let you change some of the sounds that are used by
    ## Ren'Py.

    ## Set this to False if the game does not have any sound effects.

    config.has_sound = True

    ## Set this to False if the game does not have any music.

    config.has_music = True

    ## Set this to False if the game does not have voicing.

    config.has_voice = True

    ## Sounds that are used when button and imagemaps are clicked.

    # style.button.activate_sound = "click.wav"
    # style.imagemap.activate_sound = "click.wav"

    ## Sounds that are used when entering and exiting the game menu.

    # config.enter_sound = "click.wav"
    # config.exit_sound = "click.wav"

    ## A sample sound that can be played to check the sound volume.

    # config.sample_sound = "click.wav"


    #########################################
    ## Help.

    ## This lets you configure the help option on the Ren'Py menus.
    ## It may be:
    ## - A label in the script, in which case that label is called to
    ##   show help to the user.
    ## - A file name relative to the base directory, which is opened in a
    ##   web browser.
    ## - None, to disable help.   
    config.help = "README.html"


    #########################################
    ## Transitions.

    ## Used when entering the game menu from the game.
    config.enter_transition = moveinbottom

    ## Used when exiting the game menu to the game.
    config.exit_transition = moveoutbottom

    ## Used between screens of the game menu.
    config.intra_transition = dissolve

    ## Used when entering the game menu from the main menu.
    config.main_game_transition = dissolve

    ## Used when returning to the main menu from the game.
    config.game_main_transition = fade

    ## Used when entering the main menu from the splashscreen.
    config.end_splash_transition = dissolve

    ## Used when entering the main menu after the game has ended.
    config.end_game_transition = Fade(0.5, 1.0, 0.5)

    ## Used when a game is loaded.
    config.after_load_transition = Fade(0.3, 0.1, 0.3)

    ## Used when the window is shown.
    config.window_show_transition = Dissolve(0.2)

    ## Used when the window is hidden.
    config.window_hide_transition = Dissolve(0.2)


    #########################################
    ## This is the name of the directory where the game's data is
    ## stored. (It needs to be set early, before any other init code
    ## is run, so the persisten information can be found by the init code.)
python early:
    config.save_directory = "Touhou Mecha-1310569953"

#init -1 python hide:
    #########################################
    ## Default values of Preferences.

    ## Note: These options are only evaluated the first time a
    ## game is run. To have them run a second time, delete
    ## game/saves/persistent

    # **Moved to preferences_screen.rpy**


    #########################################
    ## More customizations can go here.
    
    config.image_cache_size = 24

                         
## This section contains information about how to build your project into 
## distribution files.
init python:
    
    ## The name that's used for directories and archive files. For example, if
    ## this is 'mygame-1.0', the windows distribution will be in the
    ## directory 'mygame-1.0-win', in the 'mygame-1.0-win.zip' file.
    build.directory_name = "Touhou Mecha"
    
    ## The name that's uses for executables - the program that users will run
    ## to start the game. For example, if this is 'mygame', then on Windows,
    ## users can click 'mygame.exe' to start the game.
    build.executable_name = "Touhou Mecha"
    
    ## If True, Ren'Py will include update information into packages. This
    ## allows the updater to run.
    build.include_update = False
    
    ## File patterns:
    ## 
    ## The following functions take file patterns. File patterns are case-
    ## insensitive, and matched against the path relative to the base
    ## directory, with and without a leading /. If multiple patterns match,
    ## the first is used.
    ##    
    ##
    ## In a pattern:
    ##
    ## / 
    ##     Is the directory separator.
    ## *
    ##     Matches all characters, except the directory separator.
    ## **
    ##     Matches all characters, including the directory separator.
    ##
    ## For example:
    ##
    ## *.txt
    ##     Matches txt files in the base directory.
    ## game/**.ogg
    ##     Matches ogg files in the game directory or any of its subdirectories.
    ## **.psd
    ##    Matches psd files anywhere in the project.

    ## Classify files as None to exclude them from the built distributions.

    build.classify('**~', None)
    build.classify('**.bak', None)
    build.classify('**/.**', None)
    build.classify('**/#**', None)
    build.classify('**/thumbs.db', None)
    
    ## To archive files, classify them as 'archive'.
    
    build.classify('game/**.png', 'archive')
    build.classify('game/**.jpg', 'archive')
    build.classify('game/**.ttf', 'archive')
    build.classify('game/**.mp3', 'archive')
    build.classify('game/**.ogg', 'archive')
    build.classify('game/**.wav', 'archive')
    build.classify('game/**.py', 'archive')
    build.classify('game/**.rpy', 'archive')
    build.classify('game/**.rpyc', 'archive')

    ## Files matching documentation patterns are duplicated in a mac app
    ## build, so they appear in both the app and the zip file.

    build.documentation('*.html')
    build.documentation('*.txt')
    