init python:
    
    from copy import copy
    from operator import attrgetter
    
    if not persistent.database_entry_versions_read:
        persistent.database_entry_versions_read = dict()
    
    class DatabaseEntry(object):
        def __init__(self, category, name, faction, image_file, short_description, article_text, chapter_unlocked=None):
            self.category = category
            self.name = name
            self.faction = faction
            self.image_file = image_file
            self.short_description = short_description
            self.article_text = article_text
            self.chapter_unlocked = chapter_unlocked
            self.version_being_displayed = None
            
            self.strip_article_text_whitespace()    # Get rid of initial/final newlines
            
        def is_unlocked(self):
            if self.chapter_unlocked is None or config.developer:
                return True
            else:
                return story_chapters.has_cleared_chapter(self.chapter_unlocked)
                
        def get_unlocked_entry(self):
            entry = copy(self)
            versions = dict()
            
            entry.name, versions['name'] = DatabaseEntry.unpack_from_dict(entry.name)
            entry.faction, versions['faction'] = DatabaseEntry.unpack_from_dict(entry.faction)
            entry.image_file, versions['image'] = DatabaseEntry.unpack_from_dict(entry.image_file)
            entry.short_description, versions['desc'] = DatabaseEntry.unpack_from_dict(entry.short_description)
            entry.article_text, versions['text'] = DatabaseEntry.unpack_from_dict(entry.article_text)
            
            self.version_being_displayed = max(versions.values(), key=story_chapters.number_compare_key)
            
            return entry
            
        def get_status(self):
            """
            Get the new/updated/read status of this entry.
            """
            if not persistent.database_entry_versions_read.has_key(self.name):
                return 'new'
            elif story_chapters.number_compare_key(persistent.database_entry_versions_read[self.name]) < story_chapters.number_compare_key(self.version_being_displayed):
                return 'updated'
            else:
                return 'read'
            
        def mark_as_read(self):
            persistent.database_entry_versions_read[self.name] = self.version_being_displayed
            
        def strip_article_text_whitespace(self):
            if hasattr(self.article_text, 'keys'):
                for key in self.article_text.keys():
                    self.article_text[key] = self.article_text[key].strip()
            else:
                self.article_text = self.article_text.strip()
            
        @staticmethod
        def unpack_from_dict(item):
            if hasattr(item, 'keys'):
                # Iterate over the chapter keys from latest chapter to earliest chapter.
                for chapter_number in sorted(item.keys(), key=story_chapters.number_compare_key, reverse=True):
                    # If the chapter's been cleared, use this value for the database entry.
                    if story_chapters.has_cleared_chapter(chapter_number):
                        return item[chapter_number], chapter_number
            else:
                return item, None
        
    class MechaDatabaseEntry(DatabaseEntry):
        def __init__(self, full_name, *args, **kwargs):
            super(MechaDatabaseEntry, self).__init__('mecha', *args, **kwargs)
            self.full_name = full_name
                
        def get_unlocked_entry(self):
            entry = super(MechaDatabaseEntry, self).get_unlocked_entry()
            entry.full_name, full_name_version = DatabaseEntry.unpack_from_dict(entry.full_name)
            self.version_being_displayed = max([full_name_version, self.version_being_displayed], key=story_chapters.number_compare_key)
            return entry
            
    class CharacterDatabaseEntry(DatabaseEntry):
        def __init__(self, *args, **kwargs):
            super(CharacterDatabaseEntry, self).__init__('characters', *args, **kwargs)
            
    class GlossaryDatabaseEntry(DatabaseEntry):
        def __init__(self, *args, **kwargs):
            super(GlossaryDatabaseEntry, self).__init__('glossary', *args, **kwargs)
        
            
    class Factions(object):
        CHIREIDEN = "Chireiden"
        FREELANCE = "Freelance"
        HELL_ARMY = "Hell Army"
        MORIYA = "Moriya"
        NONE = "None"
        PARSIA = "Parsia"
        UNKNOWN = "Unknown"
        ordered = [CHIREIDEN, MORIYA, PARSIA, HELL_ARMY, FREELANCE, NONE, UNKNOWN]
            
            
    database_entries = dict()
    